#!/usr/bin/node

const minimist = require('minimist');
const fs = require('fs');
const help = fs.readFileSync(__dirname +'/help.md').toString();
const version = require('../../package.json').version;
const run = require('./run/index.js');

const command = minimist( process.argv.slice( 2 ), {
	alias: {
		// Aliases
		strict: 'useStrict',

		// Short options
		c: 'config',
		d: 'indent',
		e: 'external',
		f: 'output.format',
		g: 'globals',
		h: 'help',
		i: 'input',
		l: 'legacy',
		m: 'sourcemap',
		n: 'name',
		o: 'output.file',
		u: 'id',
		v: 'version',
		w: 'watch'
	}
});

if ( command.help || ( process.argv.length <= 2 && process.stdin.isTTY ) ) {
	console.log( `\n${help.replace('__VERSION__', version)}\n` ); // eslint-disable-line no-console
}

else if ( command.version ) {
	console.log( `rollup version ${version}` ); // eslint-disable-line no-console
}

else {
	try {
		require('source-map-support').install();
	} catch (err) {
		// do nothing
	}

	run( command );
}
