const Node = require('../Node.js');
const UNKNOWN_VALUE = require('../values.js').UNKNOWN_VALUE;

const operators = {
	'&&': ( left, right ) => left && right,
	'||': ( left, right ) => left || right
};

class LogicalExpression extends Node {
	getValue () {
		const leftValue = this.left.getValue();
		if ( leftValue === UNKNOWN_VALUE ) return UNKNOWN_VALUE;

		const rightValue = this.right.getValue();
		if ( rightValue === UNKNOWN_VALUE ) return UNKNOWN_VALUE;

		return operators[ this.operator ]( leftValue, rightValue );
	}

	hasEffectsWhenMutated ( options ) {
		const leftValue = this.left.getValue();
		if ( leftValue === UNKNOWN_VALUE ) {
			return this.left.hasEffectsWhenMutated( options ) || this.right.hasEffectsWhenMutated( options );
		}
		if ((leftValue && this.operator === '||') || (!leftValue && this.operator === '&&')) {
			return this.left.hasEffectsWhenMutated( options );
		}
		return this.right.hasEffectsWhenMutated( options );
	}
}
module.exports = LogicalExpression