const Node = require('../../Node.js');
const FunctionScope = require('../../scopes/FunctionScope');
const UNKNOWN_ASSIGNMENT = require('../../values').UNDEFINED_ASSIGNMENT;
const UNKNOWN_OBJECT_LITERAL = require('../../values').UNKNOWN_OBJECT_LITERAL;

class FunctionNode extends Node {
	bindCall ( { withNew } ) {
		const thisVariable = this.scope.findVariable( 'this' );

		if ( withNew ) {
			thisVariable.assignExpression( UNKNOWN_OBJECT_LITERAL );
		} else {
			thisVariable.assignExpression( UNKNOWN_ASSIGNMENT );
		}
	}

	hasEffects ( options ) {
		return this.included || (this.id && this.id.hasEffects( options ));
	}

	hasEffectsAsExpressionStatement ( options ) {
		return this.hasEffects( options );
	}

	hasEffectsWhenCalled ( options ) {
		const innerOptions = options.setIgnoreSafeThisMutations();
		return this.params.some( param => param.hasEffects( innerOptions ) )
			|| this.body.hasEffects( innerOptions );
	}

	hasEffectsWhenMutated () {
		return this.included;
	}

	initialiseScope ( parentScope ) {
		this.scope = new FunctionScope( { parent: parentScope } );
	}
}
module.exports = FunctionNode